@@ -81,9 +81,10 @@ impl FixedSizeBinaryArray {
 
     /// Returns a new null [`FixedSizeBinaryArray`].
     pub fn new_null(data_type: DataType, length: usize) -> Self {
+        let size = Self::maybe_get_size(&data_type).unwrap();
         Self::new(
             data_type,
-            Buffer::new_zeroed(length),
+            Buffer::new_zeroed(length * size),
             Some(Bitmap::new_zeroed(length)),
         )
     }
