@@ -61,7 +61,7 @@ impl<'a, O: Offset> GrowableBinary<'a, O> {
         let offsets = std::mem::take(&mut self.offsets);
         let values = std::mem::take(&mut self.values);
 
-        BinaryArray::<O>::from_data(data_type, offsets.into(), values.into(), validity.into())
+        BinaryArray::<O>::new(data_type, offsets.into(), values.into(), validity.into())
     }
 }
 
@@ -99,7 +99,7 @@ impl<'a, O: Offset> Growable<'a> for GrowableBinary<'a, O> {
 
 impl<'a, O: Offset> From<GrowableBinary<'a, O>> for BinaryArray<O> {
     fn from(val: GrowableBinary<'a, O>) -> Self {
-        BinaryArray::<O>::from_data(
+        BinaryArray::<O>::new(
             val.data_type,
             val.offsets.into(),
             val.values.into(),
