@@ -1,4 +1,3 @@
-#![deny(missing_docs)]
 //! Contains the [`Array`] and [`MutableArray`] trait objects declaring arrays,
 //! as well as concrete arrays (such as [`Utf8Array`] and [`MutableUtf8Array`]).
 //!
@@ -20,7 +19,6 @@
 use std::any::Any;
 
 use crate::error::Result;
-use crate::types::{days_ms, months_days_ns};
 use crate::{
     bitmap::{Bitmap, MutableBitmap},
     datatypes::DataType,
@@ -29,7 +27,7 @@ use crate::{
 pub(self) mod physical_binary;
 
 /// A trait representing an immutable Arrow array. Arrow arrays are trait objects
-/// that are infalibly downcasted to concrete types according to the [`Array::data_type`].
+/// that are infallibly downcasted to concrete types according to the [`Array::data_type`].
 pub trait Array: Send + Sync {
     /// Convert to trait object.
     fn as_any(&self) -> &dyn Any;
@@ -90,7 +88,7 @@ pub trait Array: Send + Sync {
     /// This operation is `O(1)` over `len`, as it amounts to increase two ref counts
     /// and moving the struct to the heap.
     /// # Panic
-    /// This function panics iff `offset + length >= self.len()`.
+    /// This function panics iff `offset + length > self.len()`.
     fn slice(&self, offset: usize, length: usize) -> Box<dyn Array>;
 
     /// Slices the [`Array`], returning a new `Box<dyn Array>`.
@@ -98,7 +96,7 @@ pub trait Array: Send + Sync {
     /// This operation is `O(1)` over `len`, as it amounts to increase two ref counts
     /// and moving the struct to the heap.
     /// # Safety
-    /// The caller must ensure that `offset + length <=| self.len()`
+    /// The caller must ensure that `offset + length <= self.len()`
     unsafe fn slice_unchecked(&self, offset: usize, length: usize) -> Box<dyn Array>;
 
     /// Sets the validity bitmap on this [`Array`].
@@ -351,7 +349,6 @@ impl<'a> AsRef<(dyn Array + 'a)> for dyn Array {
 mod binary;
 mod boolean;
 mod dictionary;
-mod display;
 mod fixed_size_binary;
 mod fixed_size_list;
 mod list;
@@ -365,11 +362,12 @@ mod utf8;
 
 mod equal;
 mod ffi;
+mod fmt;
 pub mod growable;
 pub mod ord;
 
-pub use display::get_display;
 pub use equal::equal;
+pub use fmt::{get_display, get_value_display};
 
 pub use crate::types::Offset;
 pub use binary::{BinaryArray, BinaryValueIter, MutableBinaryArray};
@@ -377,7 +375,7 @@ pub use boolean::{BooleanArray, MutableBooleanArray};
 pub use dictionary::{DictionaryArray, DictionaryKey, MutableDictionaryArray};
 pub use fixed_size_binary::{FixedSizeBinaryArray, MutableFixedSizeBinaryArray};
 pub use fixed_size_list::{FixedSizeListArray, MutableFixedSizeListArray};
-pub use list::{ListArray, MutableListArray};
+pub use list::{ListArray, ListValuesIter, MutableListArray};
 pub use map::MapArray;
 pub use null::NullArray;
 pub use primitive::*;
@@ -402,52 +400,6 @@ pub trait TryPush<A> {
     fn try_push(&mut self, item: A) -> Result<()>;
 }
 
-fn display_helper<T: std::fmt::Display, I: IntoIterator<Item = Option<T>>>(iter: I) -> Vec<String> {
-    iter.into_iter()
-        .map(|x| match x {
-            Some(x) => x.to_string(),
-            None => "None".to_string(),
-        })
-        .collect::<Vec<_>>()
-}
-
-fn debug_helper<T: std::fmt::Debug, I: IntoIterator<Item = Option<T>>>(iter: I) -> Vec<String> {
-    iter.into_iter()
-        .map(|x| match x {
-            Some(x) => format!("{:?}", x),
-            None => "None".to_string(),
-        })
-        .collect::<Vec<_>>()
-}
-
-fn debug_fmt<T: std::fmt::Debug, I: IntoIterator<Item = Option<T>>>(
-    iter: I,
-    head: &str,
-    f: &mut std::fmt::Formatter<'_>,
-    new_lines: bool,
-) -> std::fmt::Result {
-    let result = debug_helper(iter);
-    if new_lines {
-        write!(f, "{}[\n{}\n]", head, result.join(",\n"))
-    } else {
-        write!(f, "{}[{}]", head, result.join(", "))
-    }
-}
-
-fn display_fmt<T: std::fmt::Display, I: IntoIterator<Item = Option<T>>>(
-    iter: I,
-    head: &str,
-    f: &mut std::fmt::Formatter<'_>,
-    new_lines: bool,
-) -> std::fmt::Result {
-    let result = display_helper(iter);
-    if new_lines {
-        write!(f, "{}[\n{}\n]", head, result.join(",\n"))
-    } else {
-        write!(f, "{}[{}]", head, result.join(", "))
-    }
-}
-
 /// Trait that list arrays implement for the purposes of DRY.
 pub trait IterableListArray: Array {
     /// # Safety
