@@ -3,11 +3,13 @@ use std::sync::Arc;
 use crate::{
     bitmap::Bitmap,
     datatypes::{DataType, Field},
+    error::ArrowError,
 };
 
 use super::{new_empty_array, new_null_array, Array};
 
 mod ffi;
+pub(super) mod fmt;
 mod iterator;
 
 /// A [`StructArray`] is a nested [`Array`] with an optional validity representing
@@ -25,7 +27,7 @@ mod iterator;
 ///     Field::new("c", DataType::Int32, false),
 /// ];
 ///
-/// let array = StructArray::from_data(DataType::Struct(fields), vec![boolean, int], None);
+/// let array = StructArray::new(DataType::Struct(fields), vec![boolean, int], None);
 /// ```
 #[derive(Clone)]
 pub struct StructArray {
@@ -35,6 +37,101 @@ pub struct StructArray {
 }
 
 impl StructArray {
+    /// Returns a new [`StructArray`].
+    /// # Errors
+    /// This function errors iff:
+    /// * `data_type`'s physical type is not [`crate::datatypes::PhysicalType::Struct`].
+    /// * the children of `data_type` are empty
+    /// * the values's len is different from children's length
+    /// * any of the values's data type is different from its corresponding children' data type
+    /// * any element of values has a different length than the first element
+    /// * the validity's length is not equal to the length of the first element
+    pub fn try_new(
+        data_type: DataType,
+        values: Vec<Arc<dyn Array>>,
+        validity: Option<Bitmap>,
+    ) -> Result<Self, ArrowError> {
+        let fields = Self::try_get_fields(&data_type)?;
+        if fields.is_empty() {
+            return Err(ArrowError::oos(
+                "A StructArray must contain at least one field",
+            ));
+        }
+        if fields.len() != values.len() {
+            return Err(ArrowError::oos(
+                "A StructArray must a number of fields in its DataType equal to the number of child values",
+            ));
+        }
+
+        fields
+            .iter().map(|a| &a.data_type)
+            .zip(values.iter().map(|a| a.data_type()))
+            .enumerate()
+            .try_for_each(|(index, (data_type, child))| {
+                if data_type != child {
+                    Err(ArrowError::oos(format!(
+                        "The children DataTypes of a StructArray must equal the children data types. 
+                         However, the field {index} has data type {data_type:?} but the value has data type {child:?}"
+                    )))
+                } else {
+                    Ok(())
+                }
+            })?;
+
+        let len = values[0].len();
+        values
+            .iter()
+            .map(|a| a.len())
+            .enumerate()
+            .try_for_each(|(index, a_len)| {
+                if a_len != len {
+                    Err(ArrowError::oos(format!(
+                        "The children DataTypes of a StructArray must equal the children data types.
+                         However, the values {index} has a length of {a_len}, which is different from values 0, {len}."
+                    )))
+                } else {
+                    Ok(())
+                }
+            })?;
+
+        if validity
+            .as_ref()
+            .map_or(false, |validity| validity.len() != len)
+        {
+            return Err(ArrowError::oos(
+                "The validity length of a StructArray must match its number of elements",
+            ));
+        }
+
+        Ok(Self {
+            data_type,
+            values,
+            validity,
+        })
+    }
+
+    /// Returns a new [`StructArray`]
+    /// # Panics
+    /// This function panics iff:
+    /// * `data_type`'s physical type is not [`crate::datatypes::PhysicalType::Struct`].
+    /// * the children of `data_type` are empty
+    /// * the values's len is different from children's length
+    /// * any of the values's data type is different from its corresponding children' data type
+    /// * any element of values has a different length than the first element
+    /// * the validity's length is not equal to the length of the first element
+    pub fn new(data_type: DataType, values: Vec<Arc<dyn Array>>, validity: Option<Bitmap>) -> Self {
+        Self::try_new(data_type, values, validity).unwrap()
+    }
+
+    /// Alias for `new`
+    pub fn from_data(
+        data_type: DataType,
+        values: Vec<Arc<dyn Array>>,
+        validity: Option<Bitmap>,
+    ) -> Self {
+        Self::new(data_type, values, validity)
+    }
+
     /// Creates an empty [`StructArray`].
     pub fn new_empty(data_type: DataType) -> Self {
         if let DataType::Struct(fields) = &data_type {
@@ -42,7 +139,7 @@ impl StructArray {
                 .iter()
                 .map(|field| new_empty_array(field.data_type().clone()).into())
                 .collect();
-            Self::from_data(data_type, values, None)
+            Self::new(data_type, values, None)
         } else {
             panic!("StructArray must be initialized with DataType::Struct");
         }
@@ -55,37 +152,17 @@ impl StructArray {
                 .iter()
                 .map(|field| new_null_array(field.data_type().clone(), length).into())
                 .collect();
-            Self::from_data(data_type, values, Some(Bitmap::new_zeroed(length)))
+            Self::new(data_type, values, Some(Bitmap::new_zeroed(length)))
         } else {
             panic!("StructArray must be initialized with DataType::Struct");
         }
     }
+}
 
-    /// Canonical method to create a [`StructArray`].
-    /// # Panics
-    /// * fields are empty
-    /// * values's len is different from Fields' length.
-    /// * any element of values has a different length than the first element.
-    pub fn from_data(
-        data_type: DataType,
-        values: Vec<Arc<dyn Array>>,
-        validity: Option<Bitmap>,
-    ) -> Self {
-        let fields = Self::get_fields(&data_type);
-        assert!(!fields.is_empty());
-        assert_eq!(fields.len(), values.len());
-        assert!(values.iter().all(|x| x.len() == values[0].len()));
-        if let Some(ref validity) = validity {
-            assert_eq!(values[0].len(), validity.len());
-        }
-        Self {
-            data_type,
-            values,
-            validity,
-        }
-    }
-
+// must use
+impl StructArray {
     /// Deconstructs the [`StructArray`] into its individual components.
+    #[must_use]
     pub fn into_data(self) -> (Vec<Field>, Vec<Arc<dyn Array>>, Option<Bitmap>) {
         let Self {
             data_type,
@@ -105,6 +182,7 @@ impl StructArray {
     /// * `offset + length` must be smaller than `self.len()`.
     /// # Implementation
     /// This operation is `O(F)` where `F` is the number of fields.
+    #[must_use]
     pub fn slice(&self, offset: usize, length: usize) -> Self {
         assert!(
             offset + length <= self.len(),
@@ -118,6 +196,7 @@ impl StructArray {
     /// This operation is `O(F)` where `F` is the number of fields.
     /// # Safety
     /// The caller must ensure that `offset + length <= self.len()`.
+    #[must_use]
     pub unsafe fn slice_unchecked(&self, offset: usize, length: usize) -> Self {
         let validity = self
             .validity
@@ -137,6 +216,7 @@ impl StructArray {
     /// Sets the validity bitmap on this [`StructArray`].
     /// # Panic
     /// This function panics iff `validity.len() != self.len()`.
+    #[must_use]
     pub fn with_validity(&self, validity: Option<Bitmap>) -> Self {
         if matches!(&validity, Some(bitmap) if bitmap.len() != self.len()) {
             panic!("validity should be as least as large as the array")
@@ -173,13 +253,19 @@ impl StructArray {
 
 impl StructArray {
     /// Returns the fields the `DataType::Struct`.
-    pub fn get_fields(data_type: &DataType) -> &[Field] {
-        match data_type {
-            DataType::Struct(fields) => fields,
-            DataType::Extension(_, inner, _) => Self::get_fields(inner),
-            _ => panic!("Wrong datatype passed to Struct."),
+    pub(crate) fn try_get_fields(data_type: &DataType) -> Result<&[Field], ArrowError> {
+        match data_type.to_logical_type() {
+            DataType::Struct(fields) => Ok(fields),
+            _ => Err(ArrowError::oos(
+                "Struct array must be created with a DataType whose physical type is Struct",
+            )),
         }
     }
+
+    /// Returns the fields the `DataType::Struct`.
+    pub fn get_fields(data_type: &DataType) -> &[Field] {
+        Self::try_get_fields(data_type).unwrap()
+    }
 }
 
 impl Array for StructArray {
@@ -213,13 +299,3 @@ impl Array for StructArray {
         Box::new(self.with_validity(validity))
     }
 }
-
-impl std::fmt::Debug for StructArray {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        writeln!(f, "StructArray{{")?;
-        for (field, column) in self.fields().iter().zip(self.values()) {
-            writeln!(f, "{}: {:?},", field.name, column)?;
-        }
-        write!(f, "}}")
-    }
-}
