@@ -1,4 +1,4 @@
-use super::{Array, UnionArray};
+use super::UnionArray;
 use crate::{scalar::Scalar, trusted_len::TrustedLen};
 
 #[derive(Debug, Clone)]
