@@ -1,17 +1,21 @@
 use std::{collections::HashMap, sync::Arc};
 
 use crate::{
-    array::{display::get_value_display, display_fmt, new_empty_array, new_null_array, Array},
     bitmap::Bitmap,
     buffer::Buffer,
     datatypes::{DataType, Field, UnionMode},
+    error::ArrowError,
     scalar::{new_scalar, Scalar},
 };
 
+use super::{new_empty_array, new_null_array, Array};
+
 mod ffi;
+pub(super) mod fmt;
 mod iterator;
 
 type FieldEntry = (usize, Arc<dyn Array>);
+type UnionComponents<'a> = (&'a [Field], Option<&'a [i32]>, UnionMode);
 
 /// [`UnionArray`] represents an array whose each slot can contain different values.
 ///
@@ -35,6 +39,94 @@ pub struct UnionArray {
 }
 
 impl UnionArray {
+    /// Returns a new [`UnionArray`].
+    /// # Errors
+    /// This function errors iff:
+    /// * `data_type`'s physical type is not [`crate::datatypes::PhysicalType::Union`].
+    /// * the fields's len is different from the `data_type`'s children's length
+    /// * any of the values's data type is different from its corresponding children' data type
+    pub fn try_new(
+        data_type: DataType,
+        types: Buffer<i8>,
+        fields: Vec<Arc<dyn Array>>,
+        offsets: Option<Buffer<i32>>,
+    ) -> Result<Self, ArrowError> {
+        let (f, ids, mode) = Self::try_get_all(&data_type)?;
+
+        if f.len() != fields.len() {
+            return Err(ArrowError::oos(
+                "The number of `fields` must equal the number of children fields in DataType::Union",
+            ));
+        };
+
+        f
+            .iter().map(|a| a.data_type())
+            .zip(fields.iter().map(|a| a.data_type()))
+            .enumerate()
+            .try_for_each(|(index, (data_type, child))| {
+                if data_type != child {
+                    Err(ArrowError::oos(format!(
+                        "The children DataTypes of a UnionArray must equal the children data types. 
+                         However, the field {index} has data type {data_type:?} but the value has data type {child:?}"
+                    )))
+                } else {
+                    Ok(())
+                }
+            })?;
+
+        if offsets.is_none() != mode.is_sparse() {
+            return Err(ArrowError::oos(
+                "The offsets must be set when the Union is dense and vice-versa",
+            ));
+        }
+
+        let fields_hash = ids.as_ref().map(|ids| {
+            ids.iter()
+                .map(|x| *x as i8)
+                .enumerate()
+                .zip(fields.iter().cloned())
+                .map(|((i, type_), field)| (type_, (i, field)))
+                .collect()
+        });
+
+        // not validated:
+        // * `offsets` is valid
+        // * max id < fields.len()
+        Ok(Self {
+            data_type,
+            fields_hash,
+            fields,
+            offsets,
+            types,
+            offset: 0,
+        })
+    }
+
+    /// Returns a new [`UnionArray`].
+    /// # Panics
+    /// This function panics iff:
+    /// * `data_type`'s physical type is not [`crate::datatypes::PhysicalType::Union`].
+    /// * the fields's len is different from the `data_type`'s children's length
+    /// * any of the values's data type is different from its corresponding children' data type
+    pub fn new(
+        data_type: DataType,
+        types: Buffer<i8>,
+        fields: Vec<Arc<dyn Array>>,
+        offsets: Option<Buffer<i32>>,
+    ) -> Self {
+        Self::try_new(data_type, types, fields, offsets).unwrap()
+    }
+
+    /// Alias for `new`
+    pub fn from_data(
+        data_type: DataType,
+        types: Buffer<i8>,
+        fields: Vec<Arc<dyn Array>>,
+        offsets: Option<Buffer<i32>>,
+    ) -> Self {
+        Self::new(data_type, types, fields, offsets)
+    }
+
     /// Creates a new null [`UnionArray`].
     pub fn new_null(data_type: DataType, length: usize) -> Self {
         if let DataType::Union(f, _, mode) = &data_type {
@@ -52,7 +144,7 @@ impl UnionArray {
             // all from the same field
             let types = Buffer::new_zeroed(length);
 
-            Self::from_data(data_type, types, fields, offsets)
+            Self::new(data_type, types, fields, offsets)
         } else {
             panic!("Union struct must be created with the corresponding Union DataType")
         }
@@ -84,51 +176,9 @@ impl UnionArray {
             panic!("Union struct must be created with the corresponding Union DataType")
         }
     }
+}
 
-    /// Creates a new [`UnionArray`].
-    pub fn from_data(
-        data_type: DataType,
-        types: Buffer<i8>,
-        fields: Vec<Arc<dyn Array>>,
-        offsets: Option<Buffer<i32>>,
-    ) -> Self {
-        let (f, ids, mode) = Self::get_all(&data_type);
-
-        if f.len() != fields.len() {
-            panic!("The number of `fields` must equal the number of fields in the Union DataType")
-        };
-        let same_data_types = f
-            .iter()
-            .zip(fields.iter())
-            .all(|(f, array)| f.data_type() == array.data_type());
-        if !same_data_types {
-            panic!("All fields' datatype in the union must equal the datatypes on the fields.")
-        }
-        if offsets.is_none() != mode.is_sparse() {
-            panic!("Sparsness flag must equal to noness of offsets in UnionArray")
-        }
-        let fields_hash = ids.as_ref().map(|ids| {
-            ids.iter()
-                .map(|x| *x as i8)
-                .enumerate()
-                .zip(fields.iter().cloned())
-                .map(|((i, type_), field)| (type_, (i, field)))
-                .collect()
-        });
-
-        // not validated:
-        // * `offsets` is valid
-        // * max id < fields.len()
-        Self {
-            data_type,
-            fields_hash,
-            fields,
-            offsets,
-            types,
-            offset: 0,
-        }
-    }
-
+impl UnionArray {
     /// Returns a slice of this [`UnionArray`].
     /// # Implementation
     /// This operation is `O(F)` where `F` is the number of fields.
@@ -141,7 +191,10 @@ impl UnionArray {
             fields: self.fields.clone(),
             fields_hash: self.fields_hash.clone(),
             types: self.types.clone().slice(offset, length),
-            offsets: self.offsets.clone(),
+            offsets: self
+                .offsets
+                .clone()
+                .map(|offsets| offsets.slice(offset, length)),
             offset: self.offset + offset,
         }
     }
@@ -158,7 +211,10 @@ impl UnionArray {
             fields: self.fields.clone(),
             fields_hash: self.fields_hash.clone(),
             types: self.types.clone().slice_unchecked(offset, length),
-            offsets: self.offsets.clone(),
+            offsets: self
+                .offsets
+                .clone()
+                .map(|offsets| offsets.slice_unchecked(offset, length)),
             offset: self.offset + offset,
         }
     }
@@ -172,8 +228,8 @@ impl UnionArray {
     }
 
     /// The optional offsets.
-    pub fn offsets(&self) -> &Option<Buffer<i32>> {
-        &self.offsets
+    pub fn offsets(&self) -> Option<&Buffer<i32>> {
+        self.offsets.as_ref()
     }
 
     /// The fields.
@@ -199,7 +255,7 @@ impl UnionArray {
         self.offsets()
             .as_ref()
             .map(|x| x[index] as usize)
-            .unwrap_or(index)
+            .unwrap_or(index + self.offset)
     }
 
     /// Returns the index and slot of the field to select from `self.fields`.
@@ -252,13 +308,21 @@ impl Array for UnionArray {
 }
 
 impl UnionArray {
-    fn get_all(data_type: &DataType) -> (&[Field], Option<&[i32]>, UnionMode) {
+    fn try_get_all(data_type: &DataType) -> Result<UnionComponents, ArrowError> {
         match data_type.to_logical_type() {
-            DataType::Union(fields, ids, mode) => (fields, ids.as_ref().map(|x| x.as_ref()), *mode),
-            _ => panic!("Wrong datatype passed to UnionArray."),
+            DataType::Union(fields, ids, mode) => {
+                Ok((fields, ids.as_ref().map(|x| x.as_ref()), *mode))
+            }
+            _ => Err(ArrowError::oos(
+                "The UnionArray requires a logical type of DataType::Union",
+            )),
         }
     }
 
+    fn get_all(data_type: &DataType) -> (&[Field], Option<&[i32]>, UnionMode) {
+        Self::try_get_all(data_type).unwrap()
+    }
+
     /// Returns all fields from [`DataType::Union`].
     /// # Panic
     /// Panics iff `data_type`'s logical type is not [`DataType::Union`].
@@ -273,16 +337,3 @@ impl UnionArray {
         Self::get_all(data_type).2.is_sparse()
     }
 }
-
-impl std::fmt::Debug for UnionArray {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        let display = get_value_display(self);
-        let new_lines = false;
-        let head = "UnionArray";
-        let iter = self
-            .iter()
-            .enumerate()
-            .map(|(i, x)| if x.is_valid() { Some(display(i)) } else { None });
-        display_fmt(iter, head, f, new_lines)
-    }
-}
