@@ -1,3 +1,4 @@
+use either::Either;
 use std::iter::FromIterator;
 use std::sync::Arc;
 
@@ -107,7 +108,7 @@ impl Bitmap {
         self.null_count
     }
 
-    /// Slices `self`, offseting by `offset` and truncating up to `length` bits.
+    /// Slices `self`, offsetting by `offset` and truncating up to `length` bits.
     /// # Panic
     /// Panics iff `self.offset + offset + length >= self.bytes.len() * 8`, i.e. if the offset and `length`
     /// exceeds the allocated capacity of `self`.
@@ -125,7 +126,7 @@ impl Bitmap {
         // count the smallest chunk
         if length < self.length / 2 {
             // count the null values in the slice
-            self.null_count = count_zeros(&self.bytes, offset, length);
+            self.null_count = count_zeros(&self.bytes, self.offset + offset, length);
         } else {
             // subtract the null count of the chunks we slice off
             let start_end = self.offset + offset + length;
@@ -175,6 +176,26 @@ impl Bitmap {
     pub(crate) fn offset(&self) -> usize {
         self.offset
     }
+
+    /// Converts this [`Bitmap`] to [`MutableBitmap`], returning itself if the conversion
+    /// is not possible
+    ///
+    /// This operation returns a [`MutableBitmap`] iff:
+    /// * this [`Bitmap`] is not an offsetted slice of another [`Bitmap`]
+    /// * this [`Bitmap`] has not been cloned (i.e. [`Arc`]`::get_mut` yields [`Some`])
+    /// * this [`Bitmap`] was not imported from the c data interface (FFI)
+    pub fn into_mut(mut self) -> Either<Self, MutableBitmap> {
+        match (
+            self.offset,
+            Arc::get_mut(&mut self.bytes).and_then(|b| b.get_vec()),
+        ) {
+            (0, Some(v)) => {
+                let data = std::mem::take(v);
+                Either::Right(MutableBitmap::from_vec(data, self.length))
+            }
+            _ => Either::Left(self),
+        }
+    }
 }
 
 impl<P: AsRef<[bool]>> From<P> for Bitmap {
