@@ -4,44 +4,7 @@ use crate::types::simd::*;
 
 use super::super::min_max::SimdOrd;
 use super::super::sum::Sum;
-use super::simd_ord_int;
-
-macro_rules! simd_add {
-    ($simd:tt, $type:ty, $lanes:expr, $add:tt) => {
-        impl std::ops::AddAssign for $simd {
-            #[inline]
-            fn add_assign(&mut self, rhs: Self) {
-                for i in 0..$lanes {
-                    self[i] = <$type>::$add(self[i], rhs[i]);
-                }
-            }
-        }
-
-        impl std::ops::Add for $simd {
-            type Output = Self;
-
-            #[inline]
-            fn add(self, rhs: Self) -> Self::Output {
-                let mut result = Self::default();
-                for i in 0..$lanes {
-                    result[i] = <$type>::$add(self[i], rhs[i]);
-                }
-                result
-            }
-        }
-
-        impl Sum<$type> for $simd {
-            #[inline]
-            fn simd_sum(self) -> $type {
-                let mut reduced = <$type>::default();
-                (0..<$simd>::LANES).for_each(|i| {
-                    reduced += self[i];
-                });
-                reduced
-            }
-        }
-    };
-}
+use super::{simd_add, simd_ord_int};
 
 simd_add!(u8x64, u8, 64, wrapping_add);
 simd_add!(u16x32, u16, 32, wrapping_add);
