@@ -14,10 +14,11 @@ use std::ops::{Add, Sub};
 use num_traits::AsPrimitive;
 
 use crate::{
-    array::{Array, PrimitiveArray},
-    compute::arity::binary,
+    array::PrimitiveArray,
+    compute::arity::{binary, unary},
     datatypes::{DataType, TimeUnit},
     error::{ArrowError, Result},
+    scalar::{PrimitiveScalar, Scalar},
     temporal_conversions,
     types::{months_days_ns, NativeType},
 };
@@ -117,6 +118,31 @@ where
     binary(time, duration, time.data_type().clone(), op)
 }
 
+/// Adds a duration to a time array (Timestamp, Time and Date). The timeunit
+/// enum is used to scale correctly both arrays; adding seconds with seconds,
+/// or milliseconds with milliseconds.
+pub fn add_duration_scalar<T>(
+    time: &PrimitiveArray<T>,
+    duration: &PrimitiveScalar<i64>,
+) -> PrimitiveArray<T>
+where
+    f64: AsPrimitive<T>,
+    T: NativeType + Add<T, Output = T>,
+{
+    let scale = create_scale(time.data_type(), duration.data_type()).unwrap();
+    let duration = if let Some(duration) = duration.value() {
+        duration
+    } else {
+        return PrimitiveArray::<T>::new_null(time.data_type().clone(), time.len());
+    };
+
+    // Closure for the binary operation. The closure contains the scale
+    // required to add a duration to the timestamp array.
+    let op = move |a: T| a + (duration as f64 * scale).as_();
+
+    unary(time, op, time.data_type().clone())
+}
+
 /// Subtract a duration to a time array (Timestamp, Time and Date). The timeunit
 /// enum is used to scale correctly both arrays; adding seconds with seconds,
 /// or milliseconds with milliseconds.
@@ -173,6 +199,29 @@ where
     binary(time, duration, time.data_type().clone(), op)
 }
 
+/// Subtract a duration to a time array (Timestamp, Time and Date). The timeunit
+/// enum is used to scale correctly both arrays; adding seconds with seconds,
+/// or milliseconds with milliseconds.
+pub fn sub_duration_scalar<T>(
+    time: &PrimitiveArray<T>,
+    duration: &PrimitiveScalar<i64>,
+) -> PrimitiveArray<T>
+where
+    f64: AsPrimitive<T>,
+    T: NativeType + Sub<T, Output = T>,
+{
+    let scale = create_scale(time.data_type(), duration.data_type()).unwrap();
+    let duration = if let Some(duration) = duration.value() {
+        duration
+    } else {
+        return PrimitiveArray::<T>::new_null(time.data_type().clone(), time.len());
+    };
+
+    let op = move |a: T| a - (duration as f64 * scale).as_();
+
+    unary(time, op, time.data_type().clone())
+}
+
 /// Calculates the difference between two timestamps returning an array of type
 /// Duration. The timeunit enum is used to scale correctly both arrays;
 /// subtracting seconds with seconds, or milliseconds with milliseconds.
@@ -228,6 +277,40 @@ pub fn subtract_timestamps(
     }
 }
 
+/// Calculates the difference between two timestamps as [`DataType::Duration`] with the same time scale.
+pub fn sub_timestamps_scalar(
+    lhs: &PrimitiveArray<i64>,
+    rhs: &PrimitiveScalar<i64>,
+) -> Result<PrimitiveArray<i64>> {
+    let (scale, timeunit_a) =
+        if let (DataType::Timestamp(timeunit_a, None), DataType::Timestamp(timeunit_b, None)) =
+            (lhs.data_type(), rhs.data_type())
+        {
+            (
+                temporal_conversions::timeunit_scale(*timeunit_a, *timeunit_b),
+                timeunit_a,
+            )
+        } else {
+            return Err(ArrowError::InvalidArgumentError(
+                "sub_timestamps_scalar requires both arguments to be timestamps without timezone"
+                    .to_string(),
+            ));
+        };
+
+    let rhs = if let Some(value) = rhs.value() {
+        value
+    } else {
+        return Ok(PrimitiveArray::<i64>::new_null(
+            lhs.data_type().clone(),
+            lhs.len(),
+        ));
+    };
+
+    let op = move |a| a - (rhs as f64 * scale) as i64;
+
+    Ok(unary(lhs, op, DataType::Duration(*timeunit_a)))
+}
+
 /// Adds an interval to a [`DataType::Timestamp`].
 pub fn add_interval(
     timestamp: &PrimitiveArray<i64>,
@@ -285,3 +368,67 @@ pub fn add_interval(
         )),
     }
 }
+
+/// Adds an interval to a [`DataType::Timestamp`].
+pub fn add_interval_scalar(
+    timestamp: &PrimitiveArray<i64>,
+    interval: &PrimitiveScalar<months_days_ns>,
+) -> Result<PrimitiveArray<i64>> {
+    let interval = if let Some(interval) = interval.value() {
+        interval
+    } else {
+        return Ok(PrimitiveArray::<i64>::new_null(
+            timestamp.data_type().clone(),
+            timestamp.len(),
+        ));
+    };
+
+    match timestamp.data_type().to_logical_type() {
+        DataType::Timestamp(time_unit, Some(timezone_str)) => {
+            let time_unit = *time_unit;
+            let timezone = temporal_conversions::parse_offset(timezone_str);
+            match timezone {
+                Ok(timezone) => Ok(unary(
+                    timestamp,
+                    |timestamp| {
+                        temporal_conversions::add_interval(
+                            timestamp, time_unit, interval, &timezone,
+                        )
+                    },
+                    timestamp.data_type().clone(),
+                )),
+                #[cfg(feature = "chrono-tz")]
+                Err(_) => {
+                    let timezone = temporal_conversions::parse_offset_tz(timezone_str)?;
+                    Ok(unary(
+                        timestamp,
+                        |timestamp| {
+                            temporal_conversions::add_interval(
+                                timestamp, time_unit, interval, &timezone,
+                            )
+                        },
+                        timestamp.data_type().clone(),
+                    ))
+                }
+                #[cfg(not(feature = "chrono-tz"))]
+                _ => Err(ArrowError::InvalidArgumentError(format!(
+                    "timezone \"{}\" cannot be parsed (feature chrono-tz is not active)",
+                    timezone_str
+                ))),
+            }
+        }
+        DataType::Timestamp(time_unit, None) => {
+            let time_unit = *time_unit;
+            Ok(unary(
+                timestamp,
+                |timestamp| {
+                    temporal_conversions::add_naive_interval(timestamp, time_unit, interval)
+                },
+                timestamp.data_type().clone(),
+            ))
+        }
+        _ => Err(ArrowError::InvalidArgumentError(
+            "Adding an interval is only supported for `DataType::Timestamp`".to_string(),
+        )),
+    }
+}
