@@ -1,57 +1,56 @@
-#![deny(missing_docs)]
 //! contains FFI bindings to import and export [`Array`](crate::array::Array) via
 //! Arrow's [C Data Interface](https://arrow.apache.org/docs/format/CDataInterface.html)
 mod array;
 mod bridge;
-#[allow(clippy::module_inception)]
-mod ffi;
+mod generated;
 mod schema;
+mod stream;
 
 pub(crate) use array::try_from;
-pub(crate) use ffi::{ArrowArray, ArrowArrayRef};
+pub(crate) use array::{ArrowArrayRef, InternalArrowArray};
 
 use std::sync::Arc;
 
 use crate::array::Array;
-use crate::datatypes::Field;
+use crate::datatypes::{DataType, Field};
 use crate::error::Result;
 
-pub use ffi::Ffi_ArrowArray;
-pub use schema::Ffi_ArrowSchema;
-
 use self::schema::to_field;
 
+pub use generated::{ArrowArray, ArrowArrayStream, ArrowSchema};
+pub use stream::{export_iterator, ArrowArrayStreamReader};
+
 /// Exports an [`Arc<dyn Array>`] to the C data interface.
 /// # Safety
 /// The pointer `ptr` must be allocated and valid
-pub unsafe fn export_array_to_c(array: Arc<dyn Array>, ptr: *mut Ffi_ArrowArray) {
+pub unsafe fn export_array_to_c(array: Arc<dyn Array>, ptr: *mut ArrowArray) {
     let array = bridge::align_to_c_data_interface(array);
 
-    *ptr = Ffi_ArrowArray::new(array);
+    std::ptr::write_unaligned(ptr, ArrowArray::new(array));
 }
 
 /// Exports a [`Field`] to the C data interface.
 /// # Safety
 /// The pointer `ptr` must be allocated and valid
-pub unsafe fn export_field_to_c(field: &Field, ptr: *mut Ffi_ArrowSchema) {
-    *ptr = Ffi_ArrowSchema::new(field)
+pub unsafe fn export_field_to_c(field: &Field, ptr: *mut ArrowSchema) {
+    std::ptr::write_unaligned(ptr, ArrowSchema::new(field));
 }
 
 /// Imports a [`Field`] from the C data interface.
 /// # Safety
-/// This function is intrinsically `unsafe` and relies on a [`Ffi_ArrowSchema`]
+/// This function is intrinsically `unsafe` and relies on a [`ArrowSchema`]
 /// valid according to the [C data interface](https://arrow.apache.org/docs/format/CDataInterface.html) (FFI).
-pub unsafe fn import_field_from_c(field: &Ffi_ArrowSchema) -> Result<Field> {
+pub unsafe fn import_field_from_c(field: &ArrowSchema) -> Result<Field> {
     to_field(field)
 }
 
 /// Imports an [`Array`] from the C data interface.
 /// # Safety
-/// This function is intrinsically `unsafe` and relies on a [`Ffi_ArrowArray`]
+/// This function is intrinsically `unsafe` and relies on a [`ArrowArray`]
 /// valid according to the [C data interface](https://arrow.apache.org/docs/format/CDataInterface.html) (FFI).
 pub unsafe fn import_array_from_c(
-    array: Box<Ffi_ArrowArray>,
-    field: &Field,
+    array: Box<ArrowArray>,
+    data_type: DataType,
 ) -> Result<Box<dyn Array>> {
-    try_from(Arc::new(ArrowArray::new(array, field.clone())))
+    try_from(Arc::new(InternalArrowArray::new(array, data_type)))
 }
