@@ -1,9 +1,9 @@
 use std::collections::HashMap;
 
-use avro_rs::{Error, Schema};
+use avro_schema::Schema;
 use serde_json;
 
-use crate::error::Result;
+use crate::error::{ArrowError, Result};
 
 use super::Compression;
 
@@ -11,11 +11,13 @@ use super::Compression;
 pub(crate) fn deserialize_header(
     header: HashMap<String, Vec<u8>>,
 ) -> Result<(Schema, Option<Compression>)> {
-    let json = header
+    let schema = header
         .get("avro.schema")
-        .and_then(|bytes| serde_json::from_slice(bytes.as_ref()).ok())
-        .ok_or(Error::GetAvroSchemaFromMap)?;
-    let schema = Schema::parse(&json)?;
+        .ok_or_else(|| ArrowError::ExternalFormat("Avro schema must be present".to_string()))
+        .and_then(|bytes| {
+            serde_json::from_slice(bytes.as_ref())
+                .map_err(|e| ArrowError::ExternalFormat(e.to_string()))
+        })?;
 
     let compression = header.get("avro.codec").and_then(|bytes| {
         let bytes: &[u8] = bytes.as_ref();
