@@ -4,9 +4,9 @@ use csv_async::ByteRecord;
 
 use crate::{
     array::Array,
+    chunk::Chunk,
     datatypes::{DataType, Field},
     error::Result,
-    record_batch::RecordBatch,
 };
 
 use super::super::read_utils::{
@@ -15,6 +15,7 @@ use super::super::read_utils::{
 };
 
 impl ByteRecordGeneric for ByteRecord {
+    #[inline]
     fn get(&self, index: usize) -> Option<&[u8]> {
         self.get(index)
     }
@@ -30,7 +31,7 @@ pub fn deserialize_column(
     deserialize_column_gen(rows, column, datatype, line_number)
 }
 
-/// Deserializes rows [`ByteRecord`] into a [`RecordBatch`].
+/// Deserializes rows [`ByteRecord`] into [`Chunk`].
 /// Note that this is a convenience function: column deserialization
 /// is trivially parallelizable (e.g. rayon).
 pub fn deserialize_batch<F>(
@@ -39,7 +40,7 @@ pub fn deserialize_batch<F>(
     projection: Option<&[usize]>,
     line_number: usize,
     deserialize_column: F,
-) -> Result<RecordBatch>
+) -> Result<Chunk<Arc<dyn Array>>>
 where
     F: Fn(&[ByteRecord], usize, DataType, usize) -> Result<Arc<dyn Array>>,
 {
