@@ -1,40 +1,38 @@
-use std::collections::{HashMap, VecDeque};
+use std::collections::VecDeque;
 use std::{
     io::{Read, Seek},
     sync::Arc,
 };
 
-use arrow_format::ipc;
-use arrow_format::ipc::{Message::BodyCompression, Schema::MetadataVersion};
+use arrow_format::ipc::BodyCompressionRef;
+use arrow_format::ipc::MetadataVersion;
 
 use crate::array::*;
 use crate::datatypes::{DataType, Field, PhysicalType};
 use crate::error::Result;
+use crate::io::ipc::IpcField;
 
-use super::array::*;
-
-pub type Node<'a> = &'a ipc::Message::FieldNode;
+use super::{array::*, Dictionaries};
+use super::{IpcBuffer, Node};
 
 #[allow(clippy::too_many_arguments)]
 pub fn read<R: Read + Seek>(
     field_nodes: &mut VecDeque<Node>,
     field: &Field,
-    buffers: &mut VecDeque<&ipc::Schema::Buffer>,
+    ipc_field: &IpcField,
+    buffers: &mut VecDeque<IpcBuffer>,
     reader: &mut R,
-    dictionaries: &HashMap<usize, Arc<dyn Array>>,
+    dictionaries: &Dictionaries,
     block_offset: u64,
     is_little_endian: bool,
-    compression: Option<BodyCompression>,
+    compression: Option<BodyCompressionRef>,
     version: MetadataVersion,
 ) -> Result<Arc<dyn Array>> {
     use PhysicalType::*;
-    let data_type = field.data_type().clone();
+    let data_type = field.data_type.clone();
 
     match data_type.to_physical_type() {
-        Null => {
-            let array = read_null(field_nodes, data_type);
-            Ok(Arc::new(array))
-        }
+        Null => read_null(field_nodes, data_type).map(|x| Arc::new(x) as Arc<dyn Array>),
         Boolean => read_boolean(
             field_nodes,
             data_type,
@@ -120,6 +118,7 @@ pub fn read<R: Read + Seek>(
         List => read_list::<i32, _>(
             field_nodes,
             data_type,
+            ipc_field,
             buffers,
             reader,
             dictionaries,
@@ -132,6 +131,7 @@ pub fn read<R: Read + Seek>(
         LargeList => read_list::<i64, _>(
             field_nodes,
             data_type,
+            ipc_field,
             buffers,
             reader,
             dictionaries,
@@ -144,6 +144,7 @@ pub fn read<R: Read + Seek>(
         FixedSizeList => read_fixed_size_list(
             field_nodes,
             data_type,
+            ipc_field,
             buffers,
             reader,
             dictionaries,
@@ -156,6 +157,7 @@ pub fn read<R: Read + Seek>(
         Struct => read_struct(
             field_nodes,
             data_type,
+            ipc_field,
             buffers,
             reader,
             dictionaries,
@@ -169,7 +171,7 @@ pub fn read<R: Read + Seek>(
             match_integer_type!(key_type, |$T| {
                 read_dictionary::<$T, _>(
                     field_nodes,
-                    field,
+                    ipc_field.dictionary_id,
                     buffers,
                     reader,
                     dictionaries,
@@ -183,6 +185,7 @@ pub fn read<R: Read + Seek>(
         Union => read_union(
             field_nodes,
             data_type,
+            ipc_field,
             buffers,
             reader,
             dictionaries,
@@ -195,6 +198,7 @@ pub fn read<R: Read + Seek>(
         Map => read_map(
             field_nodes,
             data_type,
+            ipc_field,
             buffers,
             reader,
             dictionaries,
@@ -210,8 +214,8 @@ pub fn read<R: Read + Seek>(
 pub fn skip(
     field_nodes: &mut VecDeque<Node>,
     data_type: &DataType,
-    buffers: &mut VecDeque<&ipc::Schema::Buffer>,
-) {
+    buffers: &mut VecDeque<IpcBuffer>,
+) -> Result<()> {
     use PhysicalType::*;
     match data_type.to_physical_type() {
         Null => skip_null(field_nodes),
