@@ -458,6 +458,7 @@ pub fn _write_dictionary<K: DictionaryKey>(
     }
 }
 
+/// Writes a dictionary array
 #[allow(clippy::too_many_arguments)]
 pub fn write_dictionary(
     array: &dyn Array,
@@ -488,6 +489,7 @@ pub fn write_dictionary(
     }
 }
 
+/// Writes an [`Array`] to `arrow_data`
 pub fn write(
     array: &dyn Array,
     buffers: &mut Vec<ipc::Buffer>,
@@ -657,14 +659,7 @@ fn write_bytes(
         arrow_data.extend_from_slice(bytes);
     };
 
-    pad_buffer_to_8(arrow_data, arrow_data.len() - start);
-
-    let total_len = (arrow_data.len() - start) as i64;
-    buffers.push(ipc::Buffer {
-        offset: *offset,
-        length: total_len,
-    });
-    *offset += total_len;
+    buffers.push(finish_buffer(arrow_data, start, offset));
 }
 
 fn write_bitmap(
@@ -713,14 +708,7 @@ fn write_buffer<T: NativeType>(
         _write_buffer(buffer, arrow_data, is_little_endian);
     };
 
-    pad_buffer_to_8(arrow_data, arrow_data.len() - start);
-
-    let total_len = (arrow_data.len() - start) as i64;
-    buffers.push(ipc::Buffer {
-        offset: *offset,
-        length: total_len,
-    });
-    *offset += total_len;
+    buffers.push(finish_buffer(arrow_data, start, offset));
 }
 
 #[inline]
@@ -821,12 +809,19 @@ fn write_buffer_from_iter<T: NativeType, I: TrustedLen<Item = T>>(
         _write_buffer_from_iter(buffer, arrow_data, is_little_endian);
     }
 
+    buffers.push(finish_buffer(arrow_data, start, offset));
+}
+
+fn finish_buffer(arrow_data: &mut Vec<u8>, start: usize, offset: &mut i64) -> ipc::Buffer {
+    let buffer_len = (arrow_data.len() - start) as i64;
+
     pad_buffer_to_8(arrow_data, arrow_data.len() - start);
+    let total_len = (arrow_data.len() - start) as i64;
 
-    let total_len = (arrow_data.len() - start) as i64;
-    buffers.push(ipc::Buffer {
+    let buffer = ipc::Buffer {
         offset: *offset,
-        length: total_len,
-    });
+        length: buffer_len,
+    };
     *offset += total_len;
+    buffer
 }
