@@ -1,5 +1,5 @@
 use parquet2::{
-    encoding::Encoding, metadata::ColumnDescriptor, page::CompressedDataPage, write::WriteOptions,
+    encoding::Encoding, metadata::ColumnDescriptor, page::DataPage, write::WriteOptions,
 };
 
 use super::super::{levels, utils};
@@ -15,7 +15,7 @@ pub fn array_to_page<O>(
     options: WriteOptions,
     descriptor: ColumnDescriptor,
     nested: levels::NestedInfo<O>,
-) -> Result<CompressedDataPage>
+) -> Result<DataPage>
 where
     O: Offset,
 {
@@ -32,14 +32,6 @@ where
 
     encode_plain(array, is_optional, &mut buffer)?;
 
-    let uncompressed_page_size = buffer.len();
-
-    let buffer = utils::compress(
-        buffer,
-        options,
-        definition_levels_byte_length + repetition_levels_byte_length,
-    )?;
-
     let statistics = if options.write_statistics {
         Some(build_statistics(array))
     } else {
@@ -50,7 +42,6 @@ where
         buffer,
         levels::num_values(nested.offsets()),
         array.null_count(),
-        uncompressed_page_size,
         repetition_levels_byte_length,
         definition_levels_byte_length,
         statistics,
