@@ -6,6 +6,7 @@ use parquet2::{
 };
 
 use super::binary::encode_plain as binary_encode_plain;
+use super::fixed_len_bytes::encode_plain as fixed_binary_encode_plain;
 use super::primitive::encode_plain as primitive_encode_plain;
 use super::utf8::encode_plain as utf8_encode_plain;
 use crate::array::{Array, DictionaryArray, DictionaryKey, PrimitiveArray};
@@ -59,19 +60,15 @@ fn encode_keys<K: DictionaryKey>(
     // encode indices
     // compute the required number of bits
     if let Some(validity) = validity {
-        let keys = array
-            .iter()
-            .flatten()
-            .map(|x| {
-                let index = x.to_usize().unwrap();
-                // discard indices whose values are null, since they are part of the def levels.
-                if validity.get_bit(index) {
-                    Some(index as u32)
-                } else {
-                    None
-                }
-            })
-            .flatten();
+        let keys = array.iter().flatten().filter_map(|x| {
+            let index = x.to_usize().unwrap();
+            // discard indices whose values are null, since they are part of the def levels.
+            if validity.get_bit(index) {
+                Some(index as u32)
+            } else {
+                None
+            }
+        });
         let num_bits = utils::get_bit_width(keys.clone().max().unwrap_or(0) as u64) as u8;
 
         let keys = utils::ExactSizedIter::new(keys, array.len() - null_count);
@@ -172,6 +169,12 @@ pub fn array_to_pages<K: DictionaryKey>(
                     binary_encode_plain::<i64>(values, false, &mut buffer);
                     EncodedDictPage::new(buffer, values.len())
                 }
+                DataType::FixedSizeBinary(_) => {
+                    let mut buffer = vec![];
+                    let array = array.values().as_any().downcast_ref().unwrap();
+                    fixed_binary_encode_plain(array, false, &mut buffer);
+                    EncodedDictPage::new(buffer, array.len())
+                }
                 other => {
                     return Err(ArrowError::NotYetImplemented(format!(
                         "Writing dictionary arrays to parquet only support data type {:?}",
