@@ -2,19 +2,18 @@
 mod binary;
 mod boolean;
 mod dictionary;
+mod file;
 mod fixed_len_bytes;
 mod levels;
 mod primitive;
-mod record_batch;
+mod row_group;
 mod schema;
+mod sink;
 mod utf8;
 mod utils;
 
-pub mod stream;
-
 use crate::array::*;
 use crate::bitmap::Bitmap;
-use crate::buffer::Buffer;
 use crate::datatypes::*;
 use crate::error::{ArrowError, Result};
 use crate::io::parquet::read::is_type_nullable;
@@ -31,14 +30,15 @@ pub use parquet2::{
     page::{CompressedDataPage, CompressedPage, EncodedPage},
     schema::types::ParquetType,
     write::{
-        compress, write_file as parquet_write_file, Compressor, DynIter, DynStreamingIterator,
-        RowGroupIter, Version, WriteOptions,
+        compress, Compressor, DynIter, DynStreamingIterator, RowGroupIter, Version, WriteOptions,
     },
     FallibleStreamingIterator,
 };
-pub use record_batch::RowGroupIterator;
-use schema::schema_to_metadata_key;
+
+pub use file::FileWriter;
+pub use row_group::{row_group_iter, RowGroupIterator};
 pub use schema::to_parquet_type;
+pub use sink::FileSink;
 
 pub(self) fn decimal_length_from_precision(precision: usize) -> usize {
     // digits = floor(log_10(2^(8*n - 1) - 1))
@@ -61,37 +61,6 @@ pub fn to_parquet_schema(schema: &Schema) -> Result<SchemaDescriptor> {
     Ok(SchemaDescriptor::new("root".to_string(), parquet_types))
 }
 
-/// Writes
-pub fn write_file<'a, W, I>(
-    writer: &mut W,
-    row_groups: I,
-    schema: &Schema,
-    parquet_schema: SchemaDescriptor,
-    options: WriteOptions,
-    key_value_metadata: Option<Vec<KeyValue>>,
-) -> Result<u64>
-where
-    W: std::io::Write,
-    I: Iterator<Item = Result<RowGroupIter<'a, ArrowError>>>,
-{
-    let key_value_metadata = key_value_metadata
-        .map(|mut x| {
-            x.push(schema_to_metadata_key(schema));
-            x
-        })
-        .or_else(|| Some(vec![schema_to_metadata_key(schema)]));
-
-    let created_by = Some("Arrow2 - Native Rust implementation of Arrow".to_string());
-    Ok(parquet_write_file(
-        writer,
-        row_groups,
-        parquet_schema,
-        options,
-        created_by,
-        key_value_metadata,
-    )?)
-}
-
 /// Checks whether the `data_type` can be encoded as `encoding`.
 /// Note that this is whether this implementation supports it, which is a subset of
 /// what the parquet spec allows.
@@ -246,7 +215,7 @@ pub fn array_to_page(
                 values.extend_from_slice(bytes);
                 values.resize(values.len() + 8, 0);
             });
-            let array = FixedSizeBinaryArray::from_data(
+            let array = FixedSizeBinaryArray::new(
                 DataType::FixedSizeBinary(12),
                 values.into(),
                 array.validity().cloned(),
@@ -283,22 +252,26 @@ pub fn array_to_page(
                 .downcast_ref::<PrimitiveArray<i128>>()
                 .unwrap();
             if precision <= 9 {
-                let values = array.values().iter().map(|x| *x as i32);
-                let values = Buffer::from_trusted_len_iter(values);
-                let array = PrimitiveArray::<i32>::from_data(
-                    DataType::Int32,
-                    values,
-                    array.validity().cloned(),
-                );
+                let values = array
+                    .values()
+                    .iter()
+                    .map(|x| *x as i32)
+                    .collect::<Vec<_>>()
+                    .into();
+
+                let array =
+                    PrimitiveArray::<i32>::new(DataType::Int32, values, array.validity().cloned());
                 primitive::array_to_page::<i32, i32>(&array, options, descriptor)
             } else if precision <= 18 {
-                let values = array.values().iter().map(|x| *x as i64);
-                let values = Buffer::from_trusted_len_iter(values);
-                let array = PrimitiveArray::<i64>::from_data(
-                    DataType::Int64,
-                    values,
-                    array.validity().cloned(),
-                );
+                let values = array
+                    .values()
+                    .iter()
+                    .map(|x| *x as i64)
+                    .collect::<Vec<_>>()
+                    .into();
+
+                let array =
+                    PrimitiveArray::<i64>::new(DataType::Int64, values, array.validity().cloned());
                 primitive::array_to_page::<i64, i64>(&array, options, descriptor)
             } else {
                 let size = decimal_length_from_precision(precision);
