@@ -1,9 +1,11 @@
 #![doc = include_str!("doc/lib.md")]
+#![deny(missing_docs)]
 // So that we have more control over what is `unsafe` inside an `unsafe` block
 #![allow(unused_unsafe)]
 //
 #![allow(clippy::len_without_is_empty)]
 #![cfg_attr(docsrs, feature(doc_cfg))]
+#![cfg_attr(feature = "simd", feature(portable_simd))]
 
 #[macro_use]
 pub mod array;
@@ -28,3 +30,6 @@ pub mod util;
 // so that documentation gets test
 #[cfg(any(test, doctest))]
 mod docs;
+
+// re-exported because we return `Either` in our public API
+pub use either::Either;
