@@ -1,8 +1,12 @@
-pub use packed_simd::{
-    f32x16, f64x8, i16x32, i32x16, i64x8, i8x64, m16x32, m32x16, m64x8, m8x64, u16x32, u32x16,
-    u64x8, u8x64,
+pub use std::simd::{
+    f32x16, f32x8, f64x8, i16x32, i16x8, i32x16, i32x8, i64x8, i8x64, i8x8, mask32x16 as m32x16,
+    mask64x8 as m64x8, mask8x64 as m8x64, u16x32, u16x8, u32x16, u32x8, u64x8, u8x64, u8x8,
 };
 
+/// Vector of 32 16-bit masks
+#[allow(non_camel_case_types)]
+pub type m16x32 = std::simd::Mask<i16, 32>;
+
 use super::*;
 
 macro_rules! simd {
@@ -20,7 +24,7 @@ macro_rules! simd {
 
             #[inline]
             fn from_chunk(v: &[$type]) -> Self {
-                <$name>::from_slice_unaligned(v)
+                <$name>::from_slice(v)
             }
 
             #[inline]
@@ -67,28 +71,28 @@ chunk_macro!(u64, u8, u64x8, m64x8, from_chunk_u8);
 
 #[inline]
 fn from_chunk_u8(chunk: u8) -> m64x8 {
-    let idx = u64x8::new(1, 2, 4, 8, 16, 32, 64, 128);
+    let idx = u64x8::from_array([1, 2, 4, 8, 16, 32, 64, 128]);
     let vecmask = u64x8::splat(chunk as u64);
 
-    (idx & vecmask).eq(idx)
+    (idx & vecmask).lanes_eq(idx)
 }
 
 #[inline]
 fn from_chunk_u16(chunk: u16) -> m32x16 {
-    let idx = u32x16::new(
+    let idx = u32x16::from_array([
         1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768,
-    );
+    ]);
     let vecmask = u32x16::splat(chunk as u32);
 
-    (idx & vecmask).eq(idx)
+    (idx & vecmask).lanes_eq(idx)
 }
 
 #[inline]
 fn from_chunk_u32(chunk: u32) -> m16x32 {
-    let idx = u16x32::new(
+    let idx = u16x32::from_array([
         1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, 1, 2, 4, 8,
         16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768,
-    );
+    ]);
     let left = u16x32::from_chunk(&[
         1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, 32768, 0, 0, 0, 0, 0,
         0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -105,16 +109,16 @@ fn from_chunk_u32(chunk: u32) -> m16x32 {
     let vecmask1 = u16x32::splat(a1);
     let vecmask2 = u16x32::splat(a2);
 
-    (idx & left & vecmask1).eq(idx) | (idx & right & vecmask2).eq(idx)
+    (idx & left & vecmask1).lanes_eq(idx) | (idx & right & vecmask2).lanes_eq(idx)
 }
 
 #[inline]
 fn from_chunk_u64(chunk: u64) -> m8x64 {
-    let idx = u8x64::new(
+    let idx = u8x64::from_array([
         1, 2, 4, 8, 16, 32, 64, 128, 1, 2, 4, 8, 16, 32, 64, 128, 1, 2, 4, 8, 16, 32, 64, 128, 1,
         2, 4, 8, 16, 32, 64, 128, 1, 2, 4, 8, 16, 32, 64, 128, 1, 2, 4, 8, 16, 32, 64, 128, 1, 2,
         4, 8, 16, 32, 64, 128, 1, 2, 4, 8, 16, 32, 64, 128,
-    );
+    ]);
     let idxs = [
         u8x64::from_chunk(&[
             1, 2, 4, 8, 16, 32, 64, 128, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -162,7 +166,7 @@ fn from_chunk_u64(chunk: u64) -> m8x64 {
 
     let mut result = m8x64::default();
     for i in 0..8 {
-        result |= (idxs[i] & u8x64::splat(a[i])).eq(idx)
+        result |= (idxs[i] & u8x64::splat(a[i])).lanes_eq(idx)
     }
 
     result
@@ -177,7 +181,7 @@ mod tests {
         let a = 0b00000001000000010000000100000001u32;
         let a = from_chunk_u32(a);
         for i in 0..32 {
-            assert_eq!(a.extract(i), i % 8 == 0)
+            assert_eq!(a.test(i), i % 8 == 0)
         }
     }
 
@@ -186,7 +190,7 @@ mod tests {
         let a = 0b0000000100000001000000010000000100000001000000010000000100000001u64;
         let a = from_chunk_u64(a);
         for i in 0..64 {
-            assert_eq!(a.extract(i), i % 8 == 0)
+            assert_eq!(a.test(i), i % 8 == 0)
         }
     }
 }
