@@ -135,3 +135,10 @@ fn value_unchecked_with_wrong_offsets_panics() {
     // even if `0` is in bounds
     unsafe { array.value_unchecked(0) };
 }
+
+#[test]
+fn debug() {
+    let array = BinaryArray::<i32>::from(&[Some([1, 2].as_ref()), Some(&[]), None]);
+
+    assert_eq!(format!("{:?}", array), "BinaryArray[[1, 2], [], None]");
+}
