@@ -3,3 +3,6 @@
 mod read;
 #[cfg(feature = "io_avro_async")]
 mod read_async;
+mod write;
+#[cfg(feature = "io_avro_async")]
+mod write_async;
