@@ -1,18 +1,17 @@
-use std::{collections::HashMap, convert::TryFrom, fs::File, io::Read};
+use std::{collections::HashMap, fs::File, io::Read, sync::Arc};
 
 use arrow2::{
-    datatypes::Schema,
-    error::Result,
-    io::ipc::read::read_stream_metadata,
-    io::ipc::read::StreamReader,
-    io::json_integration::{to_record_batch, ArrowJson},
-    record_batch::RecordBatch,
+    array::Array, chunk::Chunk, datatypes::Schema, error::Result,
+    io::ipc::read::read_stream_metadata, io::ipc::read::StreamReader, io::ipc::IpcField,
+    io::json_integration::read, io::json_integration::ArrowJson,
 };
 
 use flate2::read::GzDecoder;
 
+type IpcRead = (Schema, Vec<IpcField>, Vec<Chunk<Arc<dyn Array>>>);
+
 /// Read gzipped JSON file
-pub fn read_gzip_json(version: &str, file_name: &str) -> Result<(Schema, Vec<RecordBatch>)> {
+pub fn read_gzip_json(version: &str, file_name: &str) -> Result<IpcRead> {
     let testdata = crate::test_util::arrow_test_data();
     let file = File::open(format!(
         "{}/arrow-ipc-stream/integration/{}/{}.json.gz",
@@ -27,7 +26,7 @@ pub fn read_gzip_json(version: &str, file_name: &str) -> Result<(Schema, Vec<Rec
 
     let schema = serde_json::to_value(arrow_json.schema).unwrap();
 
-    let schema = Schema::try_from(&schema)?;
+    let (schema, ipc_fields) = read::deserialize_schema(&schema)?;
 
     // read dictionaries
     let mut dictionaries = HashMap::new();
@@ -41,13 +40,13 @@ pub fn read_gzip_json(version: &str, file_name: &str) -> Result<(Schema, Vec<Rec
     let batches = arrow_json
         .batches
         .iter()
-        .map(|batch| to_record_batch(&schema, batch, &dictionaries))
+        .map(|batch| read::deserialize_chunk(&schema, &ipc_fields, batch, &dictionaries))
         .collect::<Result<Vec<_>>>()?;
 
-    Ok((schema, batches))
+    Ok((schema, ipc_fields, batches))
 }
 
-pub fn read_arrow_stream(version: &str, file_name: &str) -> (Schema, Vec<RecordBatch>) {
+pub fn read_arrow_stream(version: &str, file_name: &str) -> IpcRead {
     let testdata = crate::test_util::arrow_test_data();
     let mut file = File::open(format!(
         "{}/arrow-ipc-stream/integration/{}/{}.stream",
@@ -58,10 +57,12 @@ pub fn read_arrow_stream(version: &str, file_name: &str) -> (Schema, Vec<RecordB
     let metadata = read_stream_metadata(&mut file).unwrap();
     let reader = StreamReader::new(file, metadata);
 
-    let schema = reader.schema();
+    let schema = reader.metadata().schema.clone();
+    let ipc_fields = reader.metadata().ipc_schema.fields.clone();
 
     (
-        schema.as_ref().clone(),
+        schema,
+        ipc_fields,
         reader
             .map(|x| x.map(|x| x.unwrap()))
             .collect::<Result<_>>()
