@@ -1,7 +1,6 @@
 use std::io::{Cursor, Read, Seek};
 use std::sync::Arc;
 
-use arrow2::error::ArrowError;
 use arrow2::{
     array::*, bitmap::Bitmap, buffer::Buffer, chunk::Chunk, datatypes::*, error::Result,
     io::parquet::read::statistics::*, io::parquet::read::*, io::parquet::write::*,
@@ -11,32 +10,65 @@ use crate::io::ipc::read_gzip_json;
 
 mod read;
 mod write;
+mod write_async;
 
 type ArrayStats = (Arc<dyn Array>, Option<Box<dyn Statistics>>);
 
 pub fn read_column<R: Read + Seek>(
     mut reader: R,
     row_group: usize,
-    column: usize,
+    column: &str,
 ) -> Result<ArrayStats> {
     let metadata = read_metadata(&mut reader)?;
+    let schema = infer_schema(&metadata)?;
 
-    let mut reader = RecordReader::try_new(reader, Some(vec![column]), None, None, None)?;
+    let column = schema
+        .fields
+        .iter()
+        .enumerate()
+        .filter_map(|(i, f)| if f.name == column { Some(i) } else { None })
+        .next()
+        .unwrap();
 
-    let statistics = metadata.row_groups[row_group]
-        .column(column)
-        .statistics()
-        .map(|x| statistics::deserialize_statistics(x?.as_ref()))
-        .transpose()?;
+    let mut reader = FileReader::try_new(reader, Some(&[column]), None, None, None)?;
 
-    Ok((reader.next().unwrap()?.columns()[0].clone(), statistics))
+    let field = &schema.fields[column];
+
+    let mut statistics = deserialize_statistics(field, metadata.row_groups[row_group].columns())?;
+
+    Ok((
+        reader.next().unwrap()?.into_arrays().pop().unwrap(),
+        statistics.pop().unwrap(),
+    ))
 }
 
-pub fn pyarrow_nested_nullable(column: usize) -> Box<dyn Array> {
+pub fn pyarrow_nested_edge(column: &str) -> Box<dyn Array> {
+    match column {
+        "simple" => {
+            // [[0, 1]]
+            let data = [Some(vec![Some(0), Some(1)])];
+            let mut a = MutableListArray::<i32, MutablePrimitiveArray<i64>>::new();
+            a.try_extend(data).unwrap();
+            let array: ListArray<i32> = a.into();
+            Box::new(array)
+        }
+        "null" => {
+            // [None]
+            let data = [None::<Vec<Option<i64>>>];
+            let mut a = MutableListArray::<i32, MutablePrimitiveArray<i64>>::new();
+            a.try_extend(data).unwrap();
+            let array: ListArray<i32> = a.into();
+            Box::new(array)
+        }
+        _ => todo!(),
+    }
+}
+
+pub fn pyarrow_nested_nullable(column: &str) -> Box<dyn Array> {
     let offsets = Buffer::from_slice([0, 2, 2, 5, 8, 8, 11, 11, 12]);
 
     let values = match column {
-        0 => {
+        "list_int64" => {
             // [[0, 1], None, [2, None, 3], [4, 5, 6], [], [7, 8, 9], None, [10]]
             Arc::new(PrimitiveArray::<i64>::from(&[
                 Some(0),
@@ -53,7 +85,7 @@ pub fn pyarrow_nested_nullable(column: usize) -> Box<dyn Array> {
                 Some(10),
             ])) as Arc<dyn Array>
         }
-        1 | 2 => {
+        "list_int64_required" | "list_int64_required_required" => {
             // [[0, 1], None, [2, 0, 3], [4, 5, 6], [], [7, 8, 9], None, [10]]
             Arc::new(PrimitiveArray::<i64>::from(&[
                 Some(0),
@@ -70,7 +102,7 @@ pub fn pyarrow_nested_nullable(column: usize) -> Box<dyn Array> {
                 Some(10),
             ])) as Arc<dyn Array>
         }
-        3 => Arc::new(PrimitiveArray::<i16>::from(&[
+        "list_int16" => Arc::new(PrimitiveArray::<i16>::from(&[
             Some(0),
             Some(1),
             Some(2),
@@ -84,7 +116,7 @@ pub fn pyarrow_nested_nullable(column: usize) -> Box<dyn Array> {
             Some(9),
             Some(10),
         ])) as Arc<dyn Array>,
-        4 => Arc::new(BooleanArray::from(&[
+        "list_bool" => Arc::new(BooleanArray::from(&[
             Some(false),
             Some(true),
             Some(true),
@@ -110,7 +142,7 @@ pub fn pyarrow_nested_nullable(column: usize) -> Box<dyn Array> {
                 [""],
             ]
         */
-        5 => Arc::new(Utf8Array::<i32>::from(&[
+        "list_utf8" => Arc::new(Utf8Array::<i32>::from(&[
             Some("Hello".to_string()),
             Some("bbb".to_string()),
             Some("aa".to_string()),
@@ -124,7 +156,7 @@ pub fn pyarrow_nested_nullable(column: usize) -> Box<dyn Array> {
             Some("bbb".to_string()),
             Some("".to_string()),
         ])),
-        6 => Arc::new(BinaryArray::<i64>::from(&[
+        "list_large_binary" => Arc::new(BinaryArray::<i64>::from(&[
             Some(b"Hello".to_vec()),
             Some(b"bbb".to_vec()),
             Some(b"aa".to_vec()),
@@ -138,38 +170,24 @@ pub fn pyarrow_nested_nullable(column: usize) -> Box<dyn Array> {
             Some(b"bbb".to_vec()),
             Some(b"".to_vec()),
         ])),
-        7 | 8 | 9 => Arc::new(NullArray::from_data(DataType::Null, 1)),
+        "list_nested_i64"
+        | "list_nested_inner_required_i64"
+        | "list_nested_inner_required_required_i64" => {
+            Arc::new(NullArray::from_data(DataType::Null, 1))
+        }
         _ => unreachable!(),
     };
 
     match column {
-        0 | 1 | 3 | 4 | 5 | 6 => {
-            let field = match column {
-                0 => Field::new("item", DataType::Int64, true),
-                1 => Field::new("item", DataType::Int64, false),
-                3 => Field::new("item", DataType::Int16, true),
-                4 => Field::new("item", DataType::Boolean, true),
-                5 => Field::new("item", DataType::Utf8, true),
-                6 => Field::new("item", DataType::LargeBinary, true),
-                _ => unreachable!(),
-            };
-
-            let validity = Some(Bitmap::from([
-                true, false, true, true, true, true, false, true,
-            ]));
-            let data_type = DataType::List(Box::new(field));
-            Box::new(ListArray::<i32>::from_data(
-                data_type, offsets, values, validity,
-            ))
-        }
-        2 => {
+        "list_int64_required_required" => {
             // [[0, 1], [], [2, None, 3], [4, 5, 6], [], [7, 8, 9], [], [10]]
             let data_type = DataType::List(Box::new(Field::new("item", DataType::Int64, false)));
             Box::new(ListArray::<i32>::from_data(
                 data_type, offsets, values, None,
             ))
         }
-        7 => {
+        "list_nested_i64" => {
+            // [[0, 1]], None, [[2, None], [3]], [[4, 5], [6]], [], [[7], None, [9]], [[], [None], None], [[10]]
             let data = [
                 Some(vec![Some(vec![Some(0), Some(1)])]),
                 None,
@@ -186,7 +204,7 @@ pub fn pyarrow_nested_nullable(column: usize) -> Box<dyn Array> {
             let array: ListArray<i32> = a.into();
             Box::new(array)
         }
-        8 => {
+        "list_nested_inner_required_i64" => {
             let data = [
                 Some(vec![Some(vec![Some(0), Some(1)])]),
                 None,
@@ -203,7 +221,7 @@ pub fn pyarrow_nested_nullable(column: usize) -> Box<dyn Array> {
             let array: ListArray<i32> = a.into();
             Box::new(array)
         }
-        9 => {
+        "list_nested_inner_required_required_i64" => {
             let data = [
                 Some(vec![Some(vec![Some(0), Some(1)])]),
                 None,
@@ -224,11 +242,31 @@ pub fn pyarrow_nested_nullable(column: usize) -> Box<dyn Array> {
             let array: ListArray<i32> = a.into();
             Box::new(array)
         }
-        _ => unreachable!(),
+        _ => {
+            let field = match column {
+                "list_int64" => Field::new("item", DataType::Int64, true),
+                "list_int64_required" => Field::new("item", DataType::Int64, false),
+                "list_int16" => Field::new("item", DataType::Int16, true),
+                "list_bool" => Field::new("item", DataType::Boolean, true),
+                "list_utf8" => Field::new("item", DataType::Utf8, true),
+                "list_large_binary" => Field::new("item", DataType::LargeBinary, true),
+                _ => unreachable!(),
+            };
+
+            let validity = Some(Bitmap::from([
+                true, false, true, true, true, true, false, true,
+            ]));
+            // [0, 2, 2, 5, 8, 8, 11, 11, 12]
+            // [[a1, a2], None, [a3, a4, a5], [a6, a7, a8], [], [a9, a10, a11], None, [a12]]
+            let data_type = DataType::List(Box::new(field));
+            Box::new(ListArray::<i32>::from_data(
+                data_type, offsets, values, validity,
+            ))
+        }
     }
 }
 
-pub fn pyarrow_nullable(column: usize) -> Box<dyn Array> {
+pub fn pyarrow_nullable(column: &str) -> Box<dyn Array> {
     let i64_values = &[
         Some(0),
         Some(1),
@@ -243,8 +281,8 @@ pub fn pyarrow_nullable(column: usize) -> Box<dyn Array> {
     ];
 
     match column {
-        0 => Box::new(PrimitiveArray::<i64>::from(i64_values)),
-        1 => Box::new(PrimitiveArray::<f64>::from(&[
+        "int64" => Box::new(PrimitiveArray::<i64>::from(i64_values)),
+        "float64" => Box::new(PrimitiveArray::<f64>::from(&[
             Some(0.0),
             Some(1.0),
             None,
@@ -256,7 +294,7 @@ pub fn pyarrow_nullable(column: usize) -> Box<dyn Array> {
             None,
             Some(9.0),
         ])),
-        2 => Box::new(Utf8Array::<i32>::from(&[
+        "string" => Box::new(Utf8Array::<i32>::from(&[
             Some("Hello".to_string()),
             None,
             Some("aa".to_string()),
@@ -268,7 +306,7 @@ pub fn pyarrow_nullable(column: usize) -> Box<dyn Array> {
             Some("def".to_string()),
             Some("aaa".to_string()),
         ])),
-        3 => Box::new(BooleanArray::from(&[
+        "bool" => Box::new(BooleanArray::from(&[
             Some(true),
             None,
             Some(false),
@@ -280,91 +318,94 @@ pub fn pyarrow_nullable(column: usize) -> Box<dyn Array> {
             Some(true),
             Some(true),
         ])),
-        4 => Box::new(
+        "date" => Box::new(
             PrimitiveArray::<i64>::from(i64_values)
                 .to(DataType::Timestamp(TimeUnit::Millisecond, None)),
         ),
-        5 => {
+        "uint32" => {
             let values = i64_values
                 .iter()
                 .map(|x| x.map(|x| x as u32))
                 .collect::<Vec<_>>();
             Box::new(PrimitiveArray::<u32>::from(values))
         }
-        6 => {
+        "string_large" => {
             let keys = PrimitiveArray::<i32>::from([Some(0), Some(1), None, Some(1)]);
             let values = Arc::new(PrimitiveArray::<i32>::from_slice([10, 200]));
             Box::new(DictionaryArray::<i32>::from_data(keys, values))
         }
-        // decimal 9
-        7 => {
+        "decimal_9" => {
             let values = i64_values
                 .iter()
                 .map(|x| x.map(|x| x as i128))
                 .collect::<Vec<_>>();
             Box::new(PrimitiveArray::<i128>::from(values).to(DataType::Decimal(9, 0)))
         }
-        // decimal 18
-        8 => {
+        "decimal_18" => {
             let values = i64_values
                 .iter()
                 .map(|x| x.map(|x| x as i128))
                 .collect::<Vec<_>>();
             Box::new(PrimitiveArray::<i128>::from(values).to(DataType::Decimal(18, 0)))
         }
-        // decimal 26
-        9 => {
+        "decimal_26" => {
             let values = i64_values
                 .iter()
                 .map(|x| x.map(|x| x as i128))
                 .collect::<Vec<_>>();
             Box::new(PrimitiveArray::<i128>::from(values).to(DataType::Decimal(26, 0)))
         }
-        10 => Box::new(
+        "timestamp_us" => Box::new(
             PrimitiveArray::<i64>::from(i64_values)
                 .to(DataType::Timestamp(TimeUnit::Microsecond, None)),
         ),
+        "timestamp_s" => Box::new(
+            PrimitiveArray::<i64>::from(i64_values).to(DataType::Timestamp(TimeUnit::Second, None)),
+        ),
+        "timestamp_s_utc" => Box::new(PrimitiveArray::<i64>::from(i64_values).to(
+            DataType::Timestamp(TimeUnit::Second, Some("UTC".to_string())),
+        )),
         _ => unreachable!(),
     }
 }
 
-pub fn pyarrow_nullable_statistics(column: usize) -> Option<Box<dyn Statistics>> {
+pub fn pyarrow_nullable_statistics(column: &str) -> Option<Box<dyn Statistics>> {
     Some(match column {
-        0 => Box::new(PrimitiveStatistics::<i64> {
+        "int64" => Box::new(PrimitiveStatistics::<i64> {
             data_type: DataType::Int64,
             distinct_count: None,
             null_count: Some(3),
             min_value: Some(0),
             max_value: Some(9),
         }),
-        1 => Box::new(PrimitiveStatistics::<f64> {
+        "float64" => Box::new(PrimitiveStatistics::<f64> {
             data_type: DataType::Float64,
             distinct_count: None,
             null_count: Some(3),
             min_value: Some(0.0),
             max_value: Some(9.0),
         }),
-        2 => Box::new(Utf8Statistics {
+        "string" => Box::new(Utf8Statistics {
             null_count: Some(4),
             distinct_count: None,
             min_value: Some("".to_string()),
             max_value: Some("def".to_string()),
         }),
-        3 => Box::new(BooleanStatistics {
+        "bool" => Box::new(BooleanStatistics {
             null_count: Some(4),
             distinct_count: None,
 
             min_value: Some(false),
             max_value: Some(true),
         }),
-        4 => Box::new(PrimitiveStatistics::<i64> {
+        "date" => Box::new(PrimitiveStatistics::<i64> {
             data_type: DataType::Timestamp(TimeUnit::Millisecond, None),
             distinct_count: None,
             null_count: Some(3),
             min_value: Some(0),
             max_value: Some(9),
         }),
-        5 => Box::new(PrimitiveStatistics::<u32> {
+        "uint32" => Box::new(PrimitiveStatistics::<u32> {
             data_type: DataType::UInt32,
             null_count: Some(3),
             distinct_count: None,
@@ -372,42 +413,55 @@ pub fn pyarrow_nullable_statistics(column: usize) -> Option<Box<dyn Statistics>>
             min_value: Some(0),
             max_value: Some(9),
         }),
-        6 => return None,
-        // Decimal statistics
-        7 => Box::new(PrimitiveStatistics::<i128> {
+        "string_large" => return None,
+        "decimal_9" => Box::new(PrimitiveStatistics::<i128> {
             distinct_count: None,
             null_count: Some(3),
             min_value: Some(0i128),
             max_value: Some(9i128),
             data_type: DataType::Decimal(9, 0),
         }),
-        8 => Box::new(PrimitiveStatistics::<i128> {
+        "decimal_18" => Box::new(PrimitiveStatistics::<i128> {
             distinct_count: None,
             null_count: Some(3),
             min_value: Some(0i128),
             max_value: Some(9i128),
             data_type: DataType::Decimal(18, 0),
         }),
-        9 => Box::new(PrimitiveStatistics::<i128> {
+        "decimal_26" => Box::new(PrimitiveStatistics::<i128> {
             distinct_count: None,
             null_count: Some(3),
             min_value: Some(0i128),
             max_value: Some(9i128),
             data_type: DataType::Decimal(26, 0),
         }),
-        10 => Box::new(PrimitiveStatistics::<i64> {
+        "timestamp_us" => Box::new(PrimitiveStatistics::<i64> {
             data_type: DataType::Timestamp(TimeUnit::Microsecond, None),
             distinct_count: None,
             null_count: Some(3),
             min_value: Some(0),
             max_value: Some(9),
         }),
+        "timestamp_s" => Box::new(PrimitiveStatistics::<i64> {
+            data_type: DataType::Timestamp(TimeUnit::Second, None),
+            distinct_count: None,
+            null_count: Some(3),
+            min_value: Some(0),
+            max_value: Some(9),
+        }),
+        "timestamp_s_utc" => Box::new(PrimitiveStatistics::<i64> {
+            data_type: DataType::Timestamp(TimeUnit::Second, Some("UTC".to_string())),
+            distinct_count: None,
+            null_count: Some(3),
+            min_value: Some(0),
+            max_value: Some(9),
+        }),
         _ => unreachable!(),
     })
 }
 
 // these values match the values in `integration`
-pub fn pyarrow_required(column: usize) -> Box<dyn Array> {
+pub fn pyarrow_required(column: &str) -> Box<dyn Array> {
     let i64_values = &[
         Some(-256),
         Some(-1),
@@ -422,31 +476,28 @@ pub fn pyarrow_required(column: usize) -> Box<dyn Array> {
     ];
 
     match column {
-        0 => Box::new(PrimitiveArray::<i64>::from(i64_values)),
-        3 => Box::new(BooleanArray::from_slice(&[
+        "int64" => Box::new(PrimitiveArray::<i64>::from(i64_values)),
+        "bool" => Box::new(BooleanArray::from_slice(&[
             true, true, false, false, false, true, true, true, true, true,
         ])),
-        2 => Box::new(Utf8Array::<i32>::from_slice(&[
+        "string" => Box::new(Utf8Array::<i32>::from_slice(&[
             "Hello", "bbb", "aa", "", "bbb", "abc", "bbb", "bbb", "def", "aaa",
         ])),
-        // decimal 9
-        6 => {
+        "decimal_9" => {
             let values = i64_values
                 .iter()
                 .map(|x| x.map(|x| x as i128))
                 .collect::<Vec<_>>();
             Box::new(PrimitiveArray::<i128>::from(values).to(DataType::Decimal(9, 0)))
         }
-        // decimal 18
-        7 => {
+        "decimal_18" => {
             let values = i64_values
                 .iter()
                 .map(|x| x.map(|x| x as i128))
                 .collect::<Vec<_>>();
             Box::new(PrimitiveArray::<i128>::from(values).to(DataType::Decimal(18, 0)))
         }
-        // decimal 26
-        8 => {
+        "decimal_26" => {
             let values = i64_values
                 .iter()
                 .map(|x| x.map(|x| x as i128))
@@ -457,45 +508,42 @@ pub fn pyarrow_required(column: usize) -> Box<dyn Array> {
     }
 }
 
-pub fn pyarrow_required_statistics(column: usize) -> Option<Box<dyn Statistics>> {
+pub fn pyarrow_required_statistics(column: &str) -> Option<Box<dyn Statistics>> {
     Some(match column {
-        0 => Box::new(PrimitiveStatistics::<i64> {
+        "int64" => Box::new(PrimitiveStatistics::<i64> {
             data_type: DataType::Int64,
             null_count: Some(0),
             distinct_count: None,
             min_value: Some(0),
             max_value: Some(9),
         }),
-        3 => Box::new(BooleanStatistics {
+        "bool" => Box::new(BooleanStatistics {
             null_count: Some(0),
             distinct_count: None,
             min_value: Some(false),
             max_value: Some(true),
         }),
-        2 => Box::new(Utf8Statistics {
+        "string" => Box::new(Utf8Statistics {
             null_count: Some(0),
             distinct_count: None,
             min_value: Some("".to_string()),
             max_value: Some("def".to_string()),
         }),
-        // decimal_9
-        6 => Box::new(PrimitiveStatistics::<i128> {
+        "decimal_9" => Box::new(PrimitiveStatistics::<i128> {
             distinct_count: None,
             null_count: Some(0),
             min_value: Some(0i128),
             max_value: Some(9i128),
             data_type: DataType::Decimal(9, 0),
         }),
-        // decimal_18
-        7 => Box::new(PrimitiveStatistics::<i128> {
+        "decimal_18" => Box::new(PrimitiveStatistics::<i128> {
             distinct_count: None,
             null_count: Some(0),
             min_value: Some(0i128),
             max_value: Some(9i128),
             data_type: DataType::Decimal(18, 0),
         }),
-        // decimal_26
-        8 => Box::new(PrimitiveStatistics::<i128> {
+        "decimal_26" => Box::new(PrimitiveStatistics::<i128> {
             distinct_count: None,
             null_count: Some(0),
             min_value: Some(0i128),
@@ -506,28 +554,28 @@ pub fn pyarrow_required_statistics(column: usize) -> Option<Box<dyn Statistics>>
     })
 }
 
-pub fn pyarrow_nested_nullable_statistics(column: usize) -> Option<Box<dyn Statistics>> {
+pub fn pyarrow_nested_nullable_statistics(column: &str) -> Option<Box<dyn Statistics>> {
     Some(match column {
-        3 => Box::new(PrimitiveStatistics::<i16> {
+        "list_int16" => Box::new(PrimitiveStatistics::<i16> {
             data_type: DataType::Int16,
             distinct_count: None,
             null_count: Some(1),
             min_value: Some(0),
             max_value: Some(10),
         }),
-        4 => Box::new(BooleanStatistics {
+        "list_bool" => Box::new(BooleanStatistics {
             distinct_count: None,
             null_count: Some(1),
             min_value: Some(false),
             max_value: Some(true),
         }),
-        5 => Box::new(Utf8Statistics {
+        "list_utf8" => Box::new(Utf8Statistics {
             distinct_count: None,
             null_count: Some(1),
             min_value: Some("".to_string()),
             max_value: Some("def".to_string()),
         }),
-        6 => Box::new(BinaryStatistics {
+        "list_large_binary" => Box::new(BinaryStatistics {
             distinct_count: None,
             null_count: Some(1),
             min_value: Some(b"".to_vec()),
@@ -543,7 +591,27 @@ pub fn pyarrow_nested_nullable_statistics(column: usize) -> Option<Box<dyn Stati
     })
 }
 
-pub fn pyarrow_struct(column: usize) -> Box<dyn Array> {
+pub fn pyarrow_nested_edge_statistics(column: &str) -> Option<Box<dyn Statistics>> {
+    Some(match column {
+        "simple" => Box::new(PrimitiveStatistics::<i64> {
+            data_type: DataType::Int64,
+            distinct_count: None,
+            null_count: Some(0),
+            min_value: Some(0),
+            max_value: Some(1),
+        }),
+        "null" => Box::new(PrimitiveStatistics::<i64> {
+            data_type: DataType::Int64,
+            distinct_count: None,
+            null_count: Some(0),
+            min_value: None,
+            max_value: None,
+        }),
+        _ => unreachable!(),
+    })
+}
+
+pub fn pyarrow_struct(column: &str) -> Box<dyn Array> {
     let boolean = [
         Some(true),
         None,
@@ -562,7 +630,7 @@ pub fn pyarrow_struct(column: usize) -> Box<dyn Array> {
         Field::new("f2", DataType::Boolean, true),
     ];
     match column {
-        0 => {
+        "struct" => {
             let string = [
                 Some("Hello"),
                 None,
@@ -585,8 +653,8 @@ pub fn pyarrow_struct(column: usize) -> Box<dyn Array> {
                 None,
             ))
         }
-        1 => {
-            let struct_ = pyarrow_struct(0).into();
+        "struct_struct" => {
+            let struct_ = pyarrow_struct("struct").into();
             let values = vec![struct_, boolean];
             Box::new(StructArray::from_data(
                 DataType::Struct(vec![
@@ -601,15 +669,15 @@ pub fn pyarrow_struct(column: usize) -> Box<dyn Array> {
     }
 }
 
-pub fn pyarrow_struct_statistics(column: usize) -> Option<Box<dyn Statistics>> {
+pub fn pyarrow_struct_statistics(column: &str) -> Option<Box<dyn Statistics>> {
     match column {
-        0 => Some(Box::new(Utf8Statistics {
+        "struct" => Some(Box::new(BooleanStatistics {
             distinct_count: None,
-            null_count: Some(1),
-            min_value: Some("".to_string()),
-            max_value: Some("def".to_string()),
+            null_count: Some(4),
+            min_value: Some(false),
+            max_value: Some(true),
         })),
-        1 => Some(Box::new(BooleanStatistics {
+        "struct_struct" => Some(Box::new(BooleanStatistics {
             distinct_count: None,
             null_count: Some(1),
             min_value: Some(false),
@@ -627,51 +695,40 @@ fn integration_write(schema: &Schema, batches: &[Chunk<Arc<dyn Array>>]) -> Resu
         version: Version::V1,
     };
 
-    let parquet_schema = to_parquet_schema(schema)?;
-    let descritors = parquet_schema.columns().to_vec().into_iter();
+    let encodings = schema
+        .fields
+        .iter()
+        .map(|x| {
+            if let DataType::Dictionary(..) = x.data_type() {
+                Encoding::RleDictionary
+            } else {
+                Encoding::Plain
+            }
+        })
+        .collect();
 
-    let row_groups = batches.iter().map(|batch| {
-        let iterator = batch
-            .columns()
-            .iter()
-            .zip(descritors.clone())
-            .map(|(array, descriptor)| {
-                let encoding = if let DataType::Dictionary(..) = array.data_type() {
-                    Encoding::RleDictionary
-                } else {
-                    Encoding::Plain
-                };
-                array_to_pages(array.as_ref(), descriptor, options, encoding).map(|pages| {
-                    let encoded_pages = DynIter::new(pages.map(|x| Ok(x?)));
-                    let compressed_pages =
-                        Compressor::new(encoded_pages, options.compression, vec![])
-                            .map_err(ArrowError::from);
-                    DynStreamingIterator::new(compressed_pages)
-                })
-            });
-        let iterator = DynIter::new(iterator);
-        Ok(iterator)
-    });
+    let row_groups =
+        RowGroupIterator::try_new(batches.iter().cloned().map(Ok), schema, options, encodings)?;
 
-    let mut writer = Cursor::new(vec![]);
+    let writer = Cursor::new(vec![]);
 
-    write_file(
-        &mut writer,
-        row_groups,
-        schema,
-        parquet_schema,
-        options,
-        None,
-    )?;
+    let mut writer = FileWriter::try_new(writer, schema.clone(), options)?;
+
+    writer.start()?;
+    for group in row_groups {
+        let (group, len) = group?;
+        writer.write(group, len)?;
+    }
+    let (_size, writer) = writer.end(None)?;
 
     Ok(writer.into_inner())
 }
 
-type IntegrationRead = (Arc<Schema>, Vec<Chunk<Arc<dyn Array>>>);
+type IntegrationRead = (Schema, Vec<Chunk<Arc<dyn Array>>>);
 
 fn integration_read(data: &[u8]) -> Result<IntegrationRead> {
     let reader = Cursor::new(data);
-    let reader = RecordReader::try_new(reader, None, None, None, None)?;
+    let reader = FileReader::try_new(reader, None, None, None, None)?;
     let schema = reader.schema().clone();
     let batches = reader.collect::<Result<Vec<_>>>()?;
 
@@ -681,11 +738,17 @@ fn integration_read(data: &[u8]) -> Result<IntegrationRead> {
 fn test_file(version: &str, file_name: &str) -> Result<()> {
     let (schema, _, batches) = read_gzip_json(version, file_name)?;
 
+    // empty batches are not written/read from parquet and can be ignored
+    let batches = batches
+        .into_iter()
+        .filter(|x| !x.is_empty())
+        .collect::<Vec<_>>();
+
     let data = integration_write(&schema, &batches)?;
 
     let (read_schema, read_batches) = integration_read(&data)?;
 
-    assert_eq!(&schema, read_schema.as_ref());
+    assert_eq!(schema, read_schema);
     assert_eq!(batches, read_batches);
 
     Ok(())
@@ -722,26 +785,37 @@ fn arrow_type() -> Result<()> {
     let array3 = DictionaryArray::from_data(indices.clone(), std::sync::Arc::new(values));
 
     let values = BinaryArray::<i32>::from_slice([b"ab", b"ac"]);
-    let array4 = DictionaryArray::from_data(indices, std::sync::Arc::new(values));
+    let array4 = DictionaryArray::from_data(indices.clone(), std::sync::Arc::new(values));
+
+    let values = FixedSizeBinaryArray::from_data(
+        DataType::FixedSizeBinary(2),
+        vec![b'a', b'b', b'a', b'c'].into(),
+        None,
+    );
+    let array5 = DictionaryArray::from_data(indices, std::sync::Arc::new(values));
 
     let schema = Schema::from(vec![
         Field::new("a1", dt1, true),
         Field::new("a2", array2.data_type().clone(), true),
         Field::new("a3", array3.data_type().clone(), true),
         Field::new("a4", array4.data_type().clone(), true),
+        Field::new("a5", array5.data_type().clone(), true),
+        Field::new("a6", array5.data_type().clone(), false),
     ]);
     let batch = Chunk::try_new(vec![
         Arc::new(array) as Arc<dyn Array>,
         Arc::new(array2),
         Arc::new(array3),
         Arc::new(array4),
+        Arc::new(array5.clone()),
+        Arc::new(array5),
     ])?;
 
     let r = integration_write(&schema, &[batch.clone()])?;
 
     let (new_schema, new_batches) = integration_read(&r)?;
 
-    assert_eq!(new_schema.as_ref(), &schema);
+    assert_eq!(new_schema, schema);
     assert_eq!(new_batches, vec![batch]);
     Ok(())
 }
